unit GroupUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls,
  CE_Unit;

type
  TGroupForm = class(TForm)
    CloseBtn: TBitBtn;
    OpenDlg: TOpenDialog;
    GroupBox1: TGroupBox;
    NameEdit: TEdit;
    GroupBox2: TGroupBox;
    GroupNumEdit: TEdit;
    ScrollBar1: TScrollBar;
    HelpBtn: TBitBtn;
    NumInGroupGB: TGroupBox;
    ListBox1: TListBox;
    RemoveBtn: TBitBtn;
    AddBtn: TBitBtn;
    OrderBtn: TBitBtn;
    MainFileCB: TCheckBox;
    NewBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    ReloadBtn: TBitBtn;
    SaveBtn: TBitBtn;
    NextBtn: TBitBtn;
    PreBtn: TBitBtn;
    ClearBtn: TBitBtn;
    MenuItemOrderBtn: TBitBtn;
    procedure AddBtnClick(Sender: TObject);
    procedure FormShow(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure RemoveBtnClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure ReloadBtnClick(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PreBtnClick(Sender: TObject);
    procedure NameEditChange(Sender: TObject);
    procedure ScrollBar1Scroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
    procedure GroupBox1Click(Sender: TObject);
    procedure GroupBox2Click(Sender: TObject);
    procedure GroupNumEditKeyPress(Sender: TObject; var Key: Char);
    procedure GroupNumEditExit(Sender: TObject);
    procedure ListBox1KeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ListBox1MouseUp(Sender: TObject; Button: TMouseButton;
      Shift: TShiftState; X, Y: Integer);
    procedure FormHide(Sender: TObject);
    procedure OrderBtnClick(Sender: TObject);
    procedure ListBox1KeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure CloseBtnClick(Sender: TObject);
    procedure MainFileCBClick(Sender: TObject);
    procedure ListBox1Click(Sender: TObject);
    procedure MenuItemOrderBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
  private
    { Private declarations }
    CurGroup, NumOfGroups : byte;
    Saved, Modified : boolean;
    procedure clear;
    function PromptToSave : boolean;
    function PromptToChangeGroup : boolean;
    function GroupNamed : boolean;
    function ChangeCurGroup (i : byte) : boolean;
    procedure EnableAll;
    procedure ShowCurGroup;
    procedure UpdateGroupLab;
    procedure SwapItems (i1, i2 : integer);
  public
    { Public declarations }
  end;

var
  GroupForm: TGroupForm;

implementation

uses MainUnit, OrderUnit;

{$R *.DFM}

procedure TGroupForm.AddBtnClick(Sender: TObject);
var i : byte;
begin
  if ListBox1.Items.Count > 0 then
    OpenDlg.InitialDir := ExtractFilePath (ListBox1.Items[ListBox1.Items.Count - 1]);

  if OpenDlg.InitialDir = '' then
    if MainForm.FilesOpen > 0 then
      OpenDlg.InitialDir := ExtractFilePath (MainForm.FileName);

  if not OpenDlg.Execute then exit;

  i := 0;
  while i < OpenDlg.Files.Count do
    if not MainForm.CanAddTo (OpenDlg.Files[i]) then
      OpenDlg.Files.Delete (i)
    else
      if ListBox1.Items.IndexOf (OpenDlg.Files[i]) >= 0 then
        if MessageDlg ('File "' + OpenDlg.Files[i] + '" is already in group.  Do you wish to add it again?',
                       mtConfirmation, [mbYes, mbNo], 0) = mrNo then
          OpenDlg.Files.Delete (i)
        else
          inc (i)
      else
        inc (i);

  if OpenDlg.Files.Count = 0 then exit;

  ListBox1.Items.AddStrings (OpenDlg.Files);
  Modified := true;
  UpdateGroupLab;
  EnableAll;
end;

procedure TGroupForm.clear;
begin
  NameEdit.Clear;
  ListBox1.Clear;
end;

procedure TGroupForm.FormShow(Sender: TObject);
begin
  NumOfGroups := MainForm.FileOpenGroupMnu.Count;
  modified := false;{so PromptToSave isn't
                     called in NewBtn.Click}
  if MainForm.GroupOpen = 0 then
    NewBtn.Click
  else
    ChangeCurGroup (MainForm.GroupOpen);

  OpenDlg.Filter := MainForm.OpenDlg.Filter;
  NameEdit.SetFocus;
end;

procedure TGroupForm.NewBtnClick(Sender: TObject);
begin
  PromptToSave;

  if NumOfGroups > MainForm.FileOpenGroupMnu.Count then begin
    if CurGroup <> NumOfGroups then ChangeCurGroup (NumOfGroups);
    exit;
  end;

  Saved := false;
  Modified := false;
  inc (NumOfGroups);
  ScrollBar1.Max := NumOfGroups;
  NameEdit.SetFocus;
  ChangeCurGroup (NumOfGroups);
end;

function TGroupForm.PromptToChangeGroup : boolean;
begin
  result := MessageDlg ('Do you still wish to change the current record number?',
                 mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

function TGroupForm.PromptToSave : boolean;
begin
  result := true;

  if modified then
    if GroupNamed then begin
      if MessageDlg ('The current record has been modifed.  Do you wish to SAVE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        SaveBtn.Click
      else result := false;
    end else if Saved then
      if MessageDlg ('The current record has been modifed and now conatins insufficient data.  Do you wish to DELETE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        DeleteBtn.Click
      else result := false;
end;

function TGroupForm.GroupNamed : boolean;
begin
  result := (NameEdit.Text <> '')
            and (ListBox1.Items.Count > 0);
end;

procedure TGroupForm.RemoveBtnClick(Sender: TObject);
var i : byte;
begin
  i := ListBox1.Items.Count;
  while (i > 0) do begin
    dec (i);
    if ListBox1.Selected [i] then
      ListBox1.Items.Delete (i);
  end;
  Modified := true;
  UpdateGroupLab;
  EnableAll;
end;

function TGroupForm.ChangeCurGroup (i : byte) : boolean;
begin
  if (i < 1) or (i > NumOfGroups) then begin
    result := false;
    exit;
  end;

  {Current groups was modified but changes
   were not saved, so the user should not be
   prompted to reopen the current group}

  CurGroup := i;
  GroupNumEdit.Text := IntToStr (CurGroup);
  ScrollBar1.Position := CurGroup;
  ShowCurGroup;
  if CurGroup <= MainForm.FileOpenGroupMnu.Count then
    MainFileCB.Tag := MainForm.MainFile[CurGroup]
  else
    MainFileCB.Tag := -1;
  MainFileCB.Checked := false;

  result := true;
end;

procedure TGroupForm.ShowCurGroup;
begin
  if CurGroup > MainForm.FileOpenGroupMnu.Count then
    Clear
  else begin
    NameEdit.Text := CapToStr (MainForm.FileOpenGroupMnu.Items[CurGroup - 1].Caption);
    ListBox1.Items := MainForm.groups[CurGroup]^;
  end;

  UpdateGroupLab;

  Modified := false;
  Saved := CurGroup <= MainForm.FileOpenGroupMnu.Count;

  MainFileCB.Tag := MainForm.MainFile[CurGroup];

  EnableAll;
end;

procedure TGroupForm.SaveBtnClick(Sender: TObject);
var
  MenuItem : TGroupMenuItem;
  ReopenGroup : integer;
begin
  with MainForm do begin
    if GroupOpen = CurGroup then
      if MessageDlg ('The group currently open has been modified.  Do you want to reopen it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then begin
        ReopenGroup := GroupOpen;
        CloseOpenGroup;
      end else
        ReopenGroup := -1
    else
      ReopenGroup := -1;

    MenuItem := TGroupMenuItem.Create (FileOpenGroupMnu);
    MenuItem.Caption := StrToCap (NameEdit.Text, CurGroup);

    if CurGroup > FileOpenGroupMnu.Count then begin
      FileOpenGroupMnu.Add (MenuItem);
      new (groups[CurGroup]);
      groups[CurGroup]^ := TStringList.Create;
    end else begin
      FileOpenGroupMnu.Delete (CurGroup - 1);
      FileOpenGroupMnu.Insert (CurGroup - 1, MenuItem);
    end;

    groups[CurGroup]^.Assign (ListBox1.Items);
    MainFile[CurGroup] := MainFileCB.Tag;

    if ReopenGroup >= 0 then
      FileOpenGroupMnu.Items[ReopenGroup - 1].Click;

  end;//with...

  Saved := true;
  Modified := false;
  EnableAll;
end;

procedure TGroupForm.ReloadBtnClick(Sender: TObject);
begin
  NameEdit.Text := CapToStr (MainForm.FileOpenGroupMnu.Items[CurGroup - 1].Caption);
  ListBox1.Items.Assign (MainForm.groups[CurGroup]^);
  UpdateGroupLab;
  Modified := false;
  EnableAll;
end;

procedure TGroupForm.DeleteBtnClick(Sender: TObject);
var i : byte;
begin
  if MainForm.ConfirmPrompt then
    if MessageDlg ('Sure you with to DELETE this group?',
                   mtConfirmation, [mbYes, mbNo], 0)
      = mrNo then exit;

  with MainForm do begin
    if GroupOpen = CurGroup then begin
      //CloseOpenGroup;//FileCloseGroupMnu.Click;
      GroupOpen := 0;
      {update filename, i.e. take [groupname]
       out of it}
      FileName := FileName;
    end else if GroupOpen > CurGroup then
      dec (GroupOpen);

    for i := CurGroup to FileOpenGroupMnu.Count - 1 do
      groups[i]^ := groups[i + 1]^;

    dispose (groups[FileOpenGroupMnu.Count]);
    FileOpenGroupMnu.Delete (CurGroup - 1);

    for i := 0 to FileOpenGroupMnu.Count - 1 do
      FileOpenGroupMnu.Items[i].Caption :=
        StrToCap (CapToStr (FileOpenGroupMnu.Items[i].Caption), i + 1);
  end;//with...

  dec (NumOfGroups);
  ScrollBar1.Max := NumOfGroups;
  if CurGroup > NumOfGroups then
    ChangeCurGroup (NumOfGroups)
  else
    ChangeCurGroup (CurGroup);
end;

procedure TGroupForm.EnableAll;
begin
  NewBtn.Enabled := NumOfGroups < MaxMenuRows;
  PreBtn.Enabled := CurGroup > 1;
  NextBtn.Enabled := CurGroup < NumOfGroups;
  RemoveBtn.Enabled := ListBox1.SelCount > 0;
  DeleteBtn.Enabled := Saved;
  ReloadBtn.Enabled := Saved and Modified;
  SaveBtn.Enabled := Modified and GroupNamed;
  MainFileCB.Enabled := ListBox1.SelCount = 1;
  MenuItemOrderBtn.Enabled := MainForm.FileOpenGroupMnu.Count > 1;
  OrderBtn.Enabled := ListBox1.Items.Count > 1;
end;

procedure TGroupForm.NextBtnClick(Sender: TObject);
begin
  if not PromptToSave then
    if not PromptToChangeGroup then exit;

  ChangeCurGroup (CurGroup + 1);
end;

procedure TGroupForm.PreBtnClick(Sender: TObject);
begin
  if not PromptToSave then
    if not PromptToChangeGroup then exit;

  ChangeCurGroup (CurGroup - 1);
end;

procedure TGroupForm.NameEditChange(Sender: TObject);
begin
  Modified := true;
  EnableAll;
end;

procedure TGroupForm.ScrollBar1Scroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  PromptToSave;
  ChangeCurGroup (ScrollBar1.Position);
end;

procedure TGroupForm.GroupBox1Click(Sender: TObject);
begin
  NameEdit.SetFocus;
end;

procedure TGroupForm.GroupBox2Click(Sender: TObject);
begin
  GroupNumEdit.SetFocus;
end;

procedure TGroupForm.GroupNumEditKeyPress(Sender: TObject; var Key: Char);
begin
  ValidateKey (Key);

  if Key = #13 then begin
    GroupNumEditExit (GroupNumEdit);
    Key := #0; // prevents beep
  end;
end;

procedure TGroupForm.GroupNumEditExit(Sender : TObject);
begin
  if GroupNumEdit.Text = '' then
    GroupNumEdit.Text := IntToStr (CurGroup)
  else begin
    PromptToSave;
    if not ChangeCurGroup (StrToInt (GroupNumEdit.Text)) then
      GroupNumEdit.Text := IntToStr (CurGroup);
  end;
end;

procedure TGroupForm.ListBox1KeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  EnableAll;
end;

procedure TGroupForm.ListBox1MouseUp(Sender: TObject; Button: TMouseButton;
  Shift: TShiftState; X, Y: Integer);
begin
  EnableAll;
end;

procedure TGroupForm.UpdateGroupLab;
begin
  NumInGroupGB.Caption := '(' +
    IntToStr (ListBox1.Items.Count) +
    ') ' + '&Files in Current Group';
end;

procedure TGroupForm.SwapItems (i1, i2 : integer);

  function InValid (i : integer) : boolean;
  begin
    result := (i < 0) or (i >= MainForm.FileOpenGroupMnu.Count);
  end;

var
  MenuItem : TGroupMenuItem;
  group : TStringList;
begin
  if (i1 = i2) or InValid (i1) or InValid (i2) then exit;

  with MainForm do begin
    MenuItem := TGroupMenuItem.Create (FileOpenGroupMnu);
    group := TStringList.Create;

    MenuItem.Assign (FileOpenGroupMnu[i1]);
    FileOpenGroupMnu[i1].Assign (FileOpenGroupMnu[i2]);
    FileOpenGroupMnu[i2].Assign (MenuItem);

    inc (i1); inc (i2);

    if GroupOpen = i1 then
      GroupOpen := i2
    else if GroupOpen = i2 then
      GroupOpen := i1;

    group.Assign (groups[i1]^);
    groups[i1]^.Assign (groups[i2]^);
    groups[i2]^.Assign (group);

    if CurGroup = i1 then begin
      GroupNumEdit.Text := IntToStr (i2);
      ScrollBar1.Position := i2;
      CurGroup := i2;
    end else if CurGroup = i2 then begin
      GroupNumEdit.Text := IntToStr (i1);
      ScrollBar1.Position := i1;
      CurGroup := i1;
    end;
  end;
end;

procedure TGroupForm.FormHide(Sender: TObject);
begin
  PromptToSave;
end;

procedure TGroupForm.OrderBtnClick(Sender: TObject);
begin
  OrderForm.ListBox1.Items.Assign (ListBox1.Items);
  OrderForm.MainItem := MainFileCB.Tag;

  if OrderForm.ShowModal = mrCancel then exit;

  ListBox1.Items.Assign (OrderForm.ListBox1.Items);
  MainFileCB.Tag := OrderForm.MainItem;

  Modified := true;
  EnableAll;
end;

procedure TGroupForm.ListBox1KeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if (Key = VK_DELETE)
  and RemoveBtn.Enabled
  then RemoveBtn.Click;
end;

procedure TGroupForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;

procedure TGroupForm.MainFileCBClick(Sender: TObject);
begin
  with MainFileCB do
    if Checked then begin
      if MainFileCB.Tag <> ListBox1.ItemIndex then begin
        MainFileCB.Tag := ListBox1.ItemIndex;
        Modified := true;
        EnableAll;
      end;
    end else if Tag = ListBox1.ItemIndex then begin
      MainFileCB.Tag := -1;
      Modified := true;
      EnableAll;
    end;
end;

procedure TGroupForm.ListBox1Click(Sender: TObject);
begin
  MainFileCB.Checked := MainFileCB.Tag = ListBox1.ItemIndex;
end;

procedure TGroupForm.MenuItemOrderBtnClick(Sender: TObject);
var i : cardinal;
begin
  OrderForm.ListBox1.Items.Assign (MenuItemToStringList (MainForm.FileOpenGroupMnu));
  OrderForm.MainItem := -1;
  if OrderForm.ShowModal = mrCancel then exit;

  for i := 0 to MainForm.FileOpenGroupMnu.Count - 2 do
    SwapItems (i, FindCapMenuIndex (MainForm.FileOpenGroupMnu, OrderForm.ListBox1.Items[i]));
end;

procedure TGroupForm.ClearBtnClick(Sender: TObject);
begin
  Clear;
end;


procedure TGroupForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;
end;

end.

