unit PropertiesUnit;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, CE_Unit, ReuUnit, Mask, ExtCtrls;

type
  TPropertiesForm = class(TForm)
    OKBtn: TBitBtn;
    CancelBtn: TBitBtn;
    Bevel1: TBevel;
    FileNameGB: TGroupBox;
    Label1: TLabel;
    Label2: TLabel;
    Label3: TLabel;
    FileNameLab: TLabel;
    DOSFileNameLab: TLabel;
    GroupLab: TLabel;
    FileSizeGB: TGroupBox;
    Label4: TLabel;
    OpenFileSizeLab: TLabel;
    Label5: TLabel;
    DiskFileSizeLab: TLabel;
    AttributesGB: TGroupBox;
    ReadOnlyCB: TCheckBox;
    ArchiveCB: TCheckBox;
    HiddenCB: TCheckBox;
    SystemCB: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure ReadOnlyCBClick(Sender: TObject);
    procedure ArchiveCBClick(Sender: TObject);
    procedure HiddenCBClick(Sender: TObject);
    procedure SystemCBClick(Sender: TObject);
    procedure FormHide(Sender: TObject);
  private
    { Private declarations }
    function AnyAttrChanged : boolean;
    procedure ChangeTagBool (var CB : TCheckBox);
  public
    { Public declarations }
  end;

var
  PropertiesForm: TPropertiesForm;

implementation

uses MainUnit;

{$R *.DFM}

function TPropertiesForm.AnyAttrChanged : boolean;
begin
  result := (ReadOnlyCB.Tag = 1)
         or (HiddenCB.Tag = 1)
         or (ArchiveCB.Tag = 1)
         or (SystemCB.Tag = 1);
end;

procedure TPropertiesForm.ChangeTagBool (var CB : TCheckBox);
begin
  with CB do
    if Tag = 0 then
      Tag := 1
    else
      Tag := 0;
end;

procedure TPropertiesForm.FormShow(Sender: TObject);
var SearchRec : TSearchRec;
begin
  Application.ShowHint := AssignLastCall (Application.ShowHint);
  with MainForm do begin
    FileNameLab.Caption := FileName;
    CropLabel (FileNameLab);

    try
      DOSFileNameLab.Caption := ShortNameOf (FileName);
      except on Exception do
        DOSFileNameLab.Caption := 'unknown';
    end;
    CropLabel (DOSFileNameLab);

    if InCurGroup (FileName) then
      GroupLab.Caption := CapToStr (FileOpenGroupMnu.Items[GroupOpen - 1].Caption)
    else
      GroupLab.Caption := 'none';
    CropLabel (GroupLab);

    OpenFileSizeLab.Caption := IntToStr (Edit.GetTextLen);
  end;//with...

  with SearchRec do
    if FindFirst (MainForm.FileName, faAnyFile, SearchRec) = 0 then begin
      ReadOnlyCB.Checked := faReadOnly and Attr = faReadOnly;
      ArchiveCB.Checked := faArchive and Attr = faArchive;
      HiddenCB.Checked := faHidden and Attr = faHidden;
      SystemCB.Checked := faSysFile and Attr = faSysFile;

      DiskFileSizeLab.Caption := IntToStr (Size);
    end else begin
      ReadOnlyCB.Checked := false;
      ArchiveCB.Checked := false;
      HiddenCB.Checked := false;
      SystemCB.Checked := false;

      DiskFileSizeLab.Caption := '0';
    end;
  FindClose (SearchRec);

  {the CB's Tag = 0 initially, so AnyAttrChanged can
   tell if any check box has been changed}
  ReadOnlyCB.Tag := 0;
  HiddenCB.Tag := 0;
  ArchiveCB.Tag := 0;
  SystemCB.Tag := 0;
end;

procedure TPropertiesForm.OKBtnClick(Sender: TObject);
var Attr : TFileAttr;
begin
  if not AnyAttrChanged then exit;

  Attr := (ord (ReadOnlyCB.Checked) * faReadOnly)
              or (ord (HiddenCB.Checked) * faHidden)
              or (ord (SystemCB.Checked) * faSysFile)
              or (ord (ArchiveCB.Checked) * faArchive);

 if FileSetAttr (MainForm.FileName, Attr) <> 0 then
    ShowMsgDlg ('Cannot change file attributes. Perhaps the file is not yet saved on disk.', mtError);
end;

procedure TPropertiesForm.ReadOnlyCBClick(Sender: TObject);
begin
  ChangeTagBool (ReadOnlyCB);
end;

procedure TPropertiesForm.ArchiveCBClick(Sender: TObject);
begin
  ChangeTagBool (ArchiveCB);
end;

procedure TPropertiesForm.HiddenCBClick(Sender: TObject);
begin
  ChangeTagBool (HiddenCB);
end;

procedure TPropertiesForm.SystemCBClick(Sender: TObject);
begin
  ChangeTagBool (SystemCB);
end;

procedure TPropertiesForm.FormHide(Sender: TObject);
begin
  Application.ShowHint := AssignLastCall (true);
end;

end.
