unit ReplaceUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ExtCtrls, checklst,
  ComCtrls;

type
  TReplaceForm = class(TForm)
    Label1: TLabel;
    FindEdit: TEdit;
    Label2: TLabel;
    ReplaceEdit: TEdit;
    CancelBtn: TBitBtn;
    OKBtn: TBitBtn;
    GroupBox1: TGroupBox;
    Bevel1: TBevel;
    SelectedCB: TCheckBox;
    ReplaceAllCB: TCheckBox;
    WholeWordCB: TCheckBox;
    MatchCaseCB: TCheckBox;
    PromptCB: TCheckBox;
    SearchFilesCB: TCheckBox;
    FromCursorCB: TCheckBox;
    procedure FormShow(Sender: TObject);
    procedure OKBtnClick(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure Label2Click(Sender: TObject);
  private
    { Private declarations }
    procedure GetOptions (var Options : TSearchTypes);
    procedure adjust (var i : cardinal);
    function ReplaceDlgTop : cardinal;
  public
    { Public declarations }
  end;

var
  ReplaceForm: TReplaceForm;

implementation

uses MainUnit;

{$R *.DFM}

procedure TReplaceForm.adjust (var i : cardinal);
begin
  i := i - MainForm.Edit.SelLength + ReplaceEdit.GetTextLen;
end;

procedure TReplaceForm.GetOptions (var Options : TSearchTypes);
begin
  Options := [];
  if WholeWordCB.Checked then Options := Options + [stWholeWord];
  if MatchCaseCB.Checked then Options := Options + [stMatchCase];
  //if ReplaceAllCB.Checked then Options := Options + [frReplaceAll];
  //if DownCB.Checked then Options := Options + [frDown];
end;

procedure TReplaceForm.FormShow(Sender: TObject);
begin
  FindEdit.SetFocus;
  FindEdit.SelectAll;
  SelectedCB.Checked := MainForm.Edit.SelLength > 0;
  SearchFilesCB.Enabled := MainForm.FilesOpen > 1;
end;

procedure TReplaceForm.OKBtnClick(Sender: TObject);
var
  Options : TSearchTypes;
  mdButtons : TMsgDlgButtons;
  SelStart, SelLength : cardinal;
  i : byte;
  FoundOne : boolean;

  procedure ReplaceIt;
  begin
    with MainForm do begin
      if (SelLength > 0)
      and (Edit.SelStart >= SelStart)
      and (Edit.SelStart <= SelStart + SelLength) then
        adjust (SelLength);

      adjust (EndFindPosition);

      if FindPosition <= Edit.SelStart + ReplaceEdit.GetTextLen + Edit.SelLength + 1 then
        adjust (FindPosition);

      Edit.SelText := ReplaceEdit.Text;
    end;// with...
  end;// ReplaceIt
begin
  GetOptions (Options);
  Hide;
  with MainForm do begin
    FoundOne := false;

    if SearchFilesCB.Checked then
      i := FilesOpen
    else
      i := 1;

    if ReplaceAllCB.Checked then
      mdButtons := mbYesNoCancel
    else
      mdButtons := [mbYes, mbNo];

    repeat
      SelStart := Edit.SelStart;
      SelLength := Edit.SelLength;
      FindText := FindEdit.Text;
      if SelectedCB.Checked then begin
        FindPosition := Edit.SelStart;
        EndFindPosition := Edit.SelStart + Edit.SelLength - 1;
      end else begin
        if FromCursorCB.Checked then begin
          //if DownCB.Checked then begin
            FindPosition := Edit.SelStart;
            EndFindPosition := MainForm.Edit.GetTextLen - 1;
          //end else begin
          //  FindPosition := 1;
          //  EndFindPosition := Edit.SelStart + 1;
          //end
        end else begin
          FindPosition := 0;
          EndFindPosition := MainForm.Edit.GetTextLen - 1;
        end;//if FromCurSorBB...
      end;//if SelectedCB...

      while (Search (Options)) do begin
        if PromptCB.Checked then
          case MessageDlgPos ('Replace this Text?', mtConfirmation, mdButtons, 0, Screen.Width div 2 - 150, ReplaceDlgTop) of
            mrYes :
              ReplaceIt;

            mrCancel : begin
              Edit.SelStart := SelStart;
              Edit.SelLength := SelLength;

              if SearchFilesCB.Checked
              and (i < FilesOpen) then
                while (i > 0) do begin
                  dec (i);
                  ViewNextMnu.Click;
                end;

              FindText := '';
              exit;
            end;//end mrCancel
          end//end case
        else
          ReplaceIt;

        FoundOne := true;
        if not ReplaceAllCB.Checked then break;
      end;//while...
      if (i = 1) and (ReplaceAllCB.Checked) then
        ShowFoundDlg (FoundOne);

      Edit.SelStart := SelStart;
      Edit.SelLength := SelLength;

      dec (i);
      if SearchFilesCB.Checked then
        ViewNextMnu.Click;
    until (i = 0);
    FindText := '';
  end;//with MainForm...
end;

procedure TReplaceForm.Label1Click(Sender: TObject);
begin
  FindEdit.SetFocus;
end;

procedure TReplaceForm.Label2Click(Sender: TObject);
begin
  ReplaceEdit.SetFocus;
end;

function TReplaceForm.ReplaceDlgTop : cardinal;
begin
  {Determines the Y coordinate of the
   MessageDlgPos that prompts on replace,
   based on the Y coordinate for Edit's
   caret}
  result := MainForm.GetCaretY;
  if result < Screen.Height div 4 then
    result := result + Screen.Height div 4
  else
    result := result - Screen.Height div 4;
end;

end.

