unit ReuUnit;

interface

uses SysUtils, Dialogs, StdCtrls, Classes;

const
  CR = #13#10;
  PB = CR + CR;

function ChangeExt (const fName, ext : TFileName) : TFileName;
function CharToDriveNum (ch : char) : byte;
procedure CropLabel (var lab : TLabel);
function GetDriveLetter (const fName : TFileName) : char;
function GetFilterItemIndex (const item, filter : string) : integer;
function IntToBool (i : integer): boolean;
function IsWhiteSpace (ch : char) : boolean;
function LongNameOf (const fName : TFileName) : TFileName;
function LTrim (const str : string) : string;
function min (i1, i2 : integer) : integer;
function posE (ch : char; const str : string) : integer;
function posI (const subStr, str : string) : integer;
function posL (const subStr, str : string; start, stop : cardinal) : integer;
function posN (const subStr, str : string; n : integer) : integer;
function Replace (const src1, src2 : string; var dest : string) : boolean;
function ReplaceAll (const src1, src2 : string; var dest : string) : integer;
function ShortNameOf (const fName : TFileName) : TFileName;
procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
function StrCmpI (const s1, s2 : string) : boolean;
function TitleCase (const str : string) : string;
function tolower (ch : char) : char;
function toupper (ch : char) : char;

implementation

function ChangeExt (const fName, ext : TFileName): TFileName;
var i : cardinal;
begin
  i := posE ('.', fName);
  if i > 0 then
    result := copy (fName, 1, i - 1)
  else
    result := fName;

  if (ext <> '') and (copy (ext, 1, 1) <> '.') then
    result := result + '.' + ext
  else
    result := result + ext;
end;

//returns 'a' or 'A' as 1, 'b' or 'B' as 2...
function CharToDriveNum (ch : char) : byte;
begin
  result := Ord (UpCase (ch)) - 64;
end;

{if label will run off the edge of its parent, crop label
 will make the label fit on its parent and set the hint to
 it's caption; ShowHint will be true iff label was cropped}
procedure CropLabel (var lab : TLabel);
begin
  with lab do begin
    ShowHint := Left + Width > Parent.Width - 4;
    if not ShowHint then exit;
    Hint := Caption;
  end;
end;

function GetDriveLetter (const fName : TFileName) : char;
begin
  result := fName[1];
end;

{fills sl with the part of filter that has the extensions;
 the extension are left with the initial *;
 sl is assumed to have already been Created}
procedure FilterToStringList (var sl : TStringList; const filter : string);
var i1, i2, len : integer;
begin
  sl.Clear;
  len := length (filter);
  i2 := 0;

  repeat
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
    i1 := i2 + 1;
    i2 := posL ('|', filter, i1, len);
    if i2 > 0 then sl.Add (copy (filter, i1, i2 - i1));
  until i2 = 0;
end;

{GetFilterItemIndex ('*.jpeg',
 'All Files|*.*|JPEG (jpg, jpeg)|*.jpg;*.jpeg')
 returns 2;
 note JPEG (jpg, jpeg) cannot be JPEG (*.jpg, *.jpeg);
 note index is 1 based, but TStringList is 0 based}
function GetFilterItemIndex (const item, filter : string) : integer;
var sl : TStringList;
begin
  sl := TStringList.Create;
  FilterToStringList (sl, filter);
  result := sl.IndexOf (item) + 1;
  sl.Free;
end;

function IntToBool (i : integer): boolean;
begin
  result := i <> 0;
end;

function IsWhiteSpace (ch : char) : boolean;
begin
  result := ch in [' ',#9,#13,#10];
end;

{returns long file name version of fName;
 an exception will be raised if fName
 does not exist; fName can be a path}
function LongNameOf (const fName : TFileName) : TFileName;
var
  SearchRec : TSearchRec;
  i : integer;
begin
  //get rid of possible trailing \
  i := length (fName);
  if i = 0 then exit;
  if fName[i] = '\' then dec (i);
  result := copy (fName, 1, i);

  //FindFirst returns 0 iff successful
  i := FindFirst (result, faAnyFile, SearchRec);
  if i = 0 then
    result := LongNameOf (ExtractFilePath (result)) + '\' + SearchRec.Name
  else if length (result) > 2 then
    raise Exception.Create ('cannot convert "' + fName + '" to long file name');
end;

function LTrim (const str : string) : string;
var i, len : integer;
begin
  len := length (str);
  i := 1;
  while (i <= len) and ((str[i] = ' ') or (str[i] = #9)) do inc (i);
  result := copy (str, i, length (str) - i + 1);
end;

function min (i1, i2 : integer) : integer;
begin
  if i1 < i2 then
    result := i1
  else
    result := i2;
end;

{end pos; returns index of rightmost
 occurance of ch in str}
function posE (ch : char; const str : string) : integer;
var i, len : cardinal;
begin
  len := length (str);

  if len = 0 then begin
    result := 0;
    exit;
  end;

  for i := len downto 1 do
    if ch = str[i] then begin
      result := i;
      exit;
    end;

  result := 0;
end;

//case insensitive pos
function posI (const subStr, str : string) : integer;
begin
  result := pos (LowerCase (subStr), LowerCase (str));
end;

{returns 1 based position of subStr in Str if subStr is
 contained b/t start and stop; otherwise 0 is returned}
function posL (const subStr, str : string; start, stop : cardinal) : integer;
var len : cardinal;
begin
  len := length (subStr);
  if stop - start + 1 < len then begin
    result := 0;
    exit;
  end;

  for start := start to stop - len + 1 do
    if subStr[1] = Str[start] then
      if subStr = copy (Str, start, len) then begin
        result := start;
        exit;
      end;

  result := 0;
end;

//returns pos of nth occurance of subStr in str
function posN (const subStr, str : string; n : integer) : integer;
  var len : integer;

  function DoPosN (start, n : integer) : integer;
  begin
    result := posL (subStr, str, start, len);
    if (n = 1) or (result = 0) then exit;
    result := DoPosN (result + 1, n - 1);
  end;

begin
  assert (n > 0, 'posN called with n <= 0');
  len := length (str);
  result := DoPosN (1, n);
end;

function Replace (const src1, src2 : string; var dest : string) : boolean;
var i : integer;
begin
  i := pos (src1, dest);
  result := i > 0;
  if not result then exit;
  delete (dest, i, length (src1));
  insert (src2, dest, i);
end;

function ReplaceAll (const src1, src2 : string; var dest : string) : integer;
begin
  result := 0;
  while (Replace (src1, src2, dest)) do inc (result);
end;

{returns short file name version of fName;
 an exception will be raised if fName
 does not exist; fName can be a path}
function ShortNameOf (const fName : TFileName) : TFileName;
var SearchRec : TSearchRec;

  {if cAlternateFileName = cFileName, then
   cAlternateFileName sometimes = nil}
  function NotNullName : string;
  begin
    if TFileName (SearchRec.FindData.cAlternateFileName) = '' then
      result := TFileName (SearchRec.FindData.cFileName)
    else
      result := TFileName (SearchRec.FindData.cAlternateFileName);
  end;

var i : integer;
begin
  //get rid of possible trailing \
  i := length (fName);
  if i = 0 then exit;
  if fName[i] = '\' then dec (i);
  result := copy (fName, 1, i);

  //FindFirst returns 0 iff successful
  i := FindFirst (result, faAnyFile, SearchRec);
  {SearchRec.FindData.cAlternateFileName is DOS file name
   of fName as PChar without path}
  if i = 0 then
    result := ShortNameOf (ExtractFilePath (result)) + '\' + NotNullName
  else if length (result) > 2 then
    raise Exception.Create ('cannot convert "' + fName + '" to short file name')
  else result := UpperCase (result);
end;

procedure ShowMsgDlg (const msg : string; aType : TMsgDlgType);
begin
  MessageDlg (msg, aType, [mbOK], 0);
end;

function StrCmpI (const s1, s2 : string) : boolean;
begin
  result := StrIComp (PChar (s1), PChar (s2)) = 0;
end;

function TitleCase (const str : string) : string;
var len, i : integer;
begin
  result := '';
  len := length (str);
  if len = 0 then exit;

  result := result + toupper (str[1]);
  for i := 2 to len do
    if IsWhiteSpace (str[i - 1]) then
      result := result + toupper (str[i])
    else
      result := result + tolower (str[i]);
end;

function tolower (ch : char) : char;
begin
  if ch in ['A'..'Z'] then
    result := chr (ord (ch) + ord ('a') - ord ('A'))
  else
    result := ch;
end;

function toupper (ch : char) : char;
begin
  if ch in ['a'..'z'] then
    result := chr (ord (ch) - ord ('a') + ord ('A'))
  else
    result := ch;
end;

end.
