unit RunUnit;

interface

uses
  Windows, Messages, SysUtils, Classes,
  Graphics, Controls, Forms, Dialogs,
  StdCtrls, Buttons, ComCtrls, ExtCtrls,
  CE_Unit;

type
  TRunForm = class(TForm)
    OpenDlg: TOpenDialog;
    GroupBox1: TGroupBox;
    GroupBox2: TGroupBox;
    GroupBox3: TGroupBox;
    ParamsEdit: TEdit;
    RecNumEdit: TEdit;
    ScrollBar1: TScrollBar;
    CloseBtn: TBitBtn;
    ProgEdit: TEdit;
    ClearBtn: TBitBtn;
    NewBtn: TBitBtn;
    DeleteBtn: TBitBtn;
    SaveBtn: TBitBtn;
    ReloadBtn: TBitBtn;
    BrowseBtn: TBitBtn;
    SaveOrBackupRG: TRadioGroup;
    GroupBox5: TGroupBox;
    ProgNameEdit: TEdit;
    HelpBtn: TBitBtn;
    GroupBox6: TGroupBox;
    HotKey1: THotKey;
    GroupBox7: TGroupBox;
    BatFileMemo: TMemo;
    MenuItemOrderBtn: TBitBtn;
    NextBtn: TBitBtn;
    PreBtn: TBitBtn;
    procedure FormShow(Sender: TObject);
    procedure FormHide(Sender: TObject);
    procedure BrowseBtnClick(Sender: TObject);
    procedure ClearBtnClick(Sender: TObject);
    procedure NewBtnClick(Sender: TObject);
    procedure ProgEditChange(Sender: TObject);
    procedure DeleteBtnClick(Sender: TObject);
    procedure NextBtnClick(Sender: TObject);
    procedure PreBtnClick(Sender: TObject);
    procedure ProgNameEditChange(Sender: TObject);
    procedure SaveAllCBClick(Sender: TObject);
    procedure ParamsEditChange(Sender: TObject);
    procedure CurFileParamCBClick(Sender: TObject);
    procedure SaveBtnClick(Sender: TObject);
    procedure GroupBox1Click(Sender: TObject);
    procedure GroupBox2Click(Sender: TObject);
    procedure Label1Click(Sender: TObject);
    procedure Label2Click(Sender: TObject);
    procedure ParamsPromptCBClick(Sender: TObject);
    procedure GroupBox5Click(Sender: TObject);
    procedure FormKeyUp(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure cdCBClick(Sender: TObject);
    procedure ReloadBtnClick(Sender: TObject);
    procedure RecNumEditExit(Sender: TObject);
    procedure RecNumEditKeyPress(Sender: TObject; var Key: Char);
    procedure SaveOrBackupRGClick(Sender: TObject);
    procedure FileNameParamRGClick(Sender: TObject);
    procedure CloseBtnClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure BatEditChange(Sender: TObject);
    procedure BatFileMemoChange(Sender: TObject);
    procedure BatFileMemoKeyPress(Sender: TObject; var Key: Char);
    procedure MenuItemOrderBtnClick(Sender: TObject);
    procedure ScrollBar1Scroll(Sender: TObject; ScrollCode: TScrollCode;
      var ScrollPos: Integer);
  private
    { Private declarations }
    Saved, Modified : boolean;
    CurRec : byte;
    function PromptToChangeRec : boolean;
    procedure ChangeModify (b : boolean);
    function ChangeRecNum (i : byte) : boolean;
    function ProgNamed : boolean;
    procedure ShowCurRec;
    function PromptToSave : boolean;
    procedure SwapItems (i1, i2 : integer);
  public
    { Public declarations }
  end;

var
  RunForm: TRunForm;

implementation

uses MainUnit, Menus, OrderUnit, ReuUnit;

{$R *.DFM}

function TRunForm.PromptToChangeRec : boolean;
begin
  result := MessageDlg ('Do you still wish to change the current record number?',
                 mtConfirmation, [mbYes, mbNo], 0) = mrYes;
end;

procedure TRunForm.FormShow(Sender: TObject);
begin
  OpenDlg.InitialDir := MainForm.SaveDlg.InitialDir;
  CurRec := MainForm.NumOfRecs;
  Saved := false;
  Modified := false;
  NewBtn.Click;
  ProgEdit.SetFocus;
end;

procedure TRunForm.FormHide(Sender: TObject);
begin
  PromptToSave;
  MainForm.NumOfRecs := MainForm.RunMnu.Count;
end;

procedure TRunForm.BrowseBtnClick(Sender: TObject);
var s : string;
begin
  s := ExtractFilePath (ProgEdit.Text);
  if s <> '' then
    OpenDlg.InitialDir := s;

  if OpenDlg.Execute then
    ProgEdit.Text := OpenDlg.FileName;
end;

procedure TRunForm.ClearBtnClick(Sender: TObject);
begin
  SaveOrBackupRG.ItemIndex := 4;
  BatFileMemo.Clear;
  ProgEdit.Text := '';
  ParamsEdit.Text := '';
  ProgNameEdit.Text := '';
  HotKey1.HotKey := ShortCut (0, []);
  Modified := Saved;
end;

procedure TRunForm.ChangeModify (b : boolean);
begin
  MenuItemOrderBtn.Enabled := MainForm.RunMnu.Count > 1;
  Modified := b;
  if Modified then begin
    SaveBtn.Enabled := ProgNamed;
    DeleteBtn.Enabled := Saved;
    ReloadBtn.Enabled := CurRec <= MainForm.RunMnu.Count;
  end else begin
    SaveBtn.Enabled := false;
    DeleteBtn.Enabled := Saved;
    ReloadBtn.Enabled := false;
  end;
end;

procedure TRunForm.NewBtnClick(Sender: TObject);
begin
  PromptToSave;
  with MainForm do begin
    if NumOfRecs > RunMnu.Count then begin
      ChangeRecNum (NumOfRecs);
      exit;
    end;

    inc (NumOfRecs);
    ScrollBar1.Max := NumOfRecs;
    ChangeRecNum (NumOfRecs);
    ClearBtn.Click;
    if (CurRec = RunMnu.Count + 1)
    or (CurRec = MAX_REC_NUM) then
      NewBtn.Enabled := false;

    Saved := false;
  end;//with...
  ChangeModify (false);
end;

function TRunForm.ChangeRecNum (i : byte) : boolean;
begin
  result := (i >= 1) and (i <= MainForm.NumOfRecs);
  if not result then exit;

  CurRec := i;

  if ScrollBar1.Position <> CurRec then
    ScrollBar1.Position := CurRec;
  if Int (RecNumEdit.Text) <> CurRec then
    RecNumEdit.Text := IntToStr (CurRec);

  ShowCurRec;
end;

procedure TRunForm.ProgEditChange(Sender: TObject);
begin
  ProgNameEdit.Text := ChangeExt (ExtractFileName (ProgEdit.Text), '');
  {change modify must be called b/c if changes occurred
   after extension, the assignment above won't call
   ChagenModify}
  ChangeModify (true);
end;

procedure TRunForm.DeleteBtnClick(Sender: TObject);
var i : byte;
begin
  if MainForm.ConfirmPrompt then
    if MessageDlg ('Are you sure you want to Delete this Record?',
                   mtConfirmation, [mbYes, mbNo], 0) = mrNo then exit;

  with MainForm do begin
    RunMnu.Delete (CurRec - 1);
    for i := CurRec to RunMnu.Count do begin
      RunMnu.Items[i - 1].Caption := StrToCap (CapToStr (RunMnu.Items[i - 1].Caption), i);
      RunMnu.Tag := i;
      Progs[i]^ := Progs[i + 1]^;
    end;

    Dispose (Progs[RunMnu.Count + 1]);

    dec (NumOfRecs);
    ScrollBar1.Max := NumOfRecs;
    if RunMnu.Count = 0 then
       RunMnu.Enabled := false;

    if CurRec > NumOfRecs then
      ChangeRecNum (NumOfRecs)
    else
      ShowCurRec;

  end;//with...
end;

procedure TRunForm.NextBtnClick(Sender: TObject);
begin
  if not PromptToSave then
    if not PromptToChangeRec then exit;

  ChangeRecNum (CurRec + 1);
end;

procedure TRunForm.PreBtnClick(Sender: TObject);
begin
  if not PromptToSave then
    if not PromptToChangeRec then exit;

  ChangeRecNum (CurRec - 1);
end;

procedure TRunForm.ProgNameEditChange(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.SaveAllCBClick(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.ParamsEditChange(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.CurFileParamCBClick(Sender: TObject);
begin
  ChangeModify (true);
end;

function TRunForm.ProgNamed : boolean;
begin
  if (ProgEdit.Text <> '')
  and (ProgNameEdit.Text <> '') then
    result := true
  else
    result := false;
end;

procedure TRunForm.SaveBtnClick(Sender: TObject);
var MenuItem : TRunMenuItem;
begin
  MenuItem := TRunMenuItem.Create (MainForm.RunMnu);
  MenuItem.ShortCut := HotKey1.HotKey;
  MenuItem.Caption := StrToCap (ProgNameEdit.Text, CurRec);

  MenuItem.Tag := CurRec;

  with MainForm do begin
    if CurRec > RunMnu.Count then
      RunMnu.Add (MenuItem)
    else begin
      RunMnu.Insert (CurRec - 1, MenuItem);
      RunMnu.Delete (CurRec);
    end;

    RunMnu.Enabled := true;

    New (Progs[CurRec]);
    Progs[CurRec]^.Params := ParamsEdit.Text;
    Progs[CurRec]^.Prog := ProgEdit.Text;
    Progs[CurRec]^.SaveOrBackupIndex := SaveOrBackupRG.ItemIndex;
    Progs[CurRec]^.BatFile := TStringList.Create;
    Progs[CurRec]^.BatFile.Assign (BatFileMemo.Lines);
  end;

  Saved := true;
  NewBtn.Enabled := CurRec < MAX_REC_NUM;
  ChangeModify (false);
end;

procedure TRunForm.ShowCurRec;
var s : string;
begin
  With MainForm do begin
    if CurRec = RunMnu.Count + 1 then begin
      ClearBtn.Click;
      Saved := false;
      NewBtn.Enabled := false;
    end else begin
      ProgEdit.Text := Progs[CurRec]^.Prog;

      s := CapToStr (RunMnu.Items[CurRec - 1].Caption);

      ParamsEdit.Text := Progs[CurRec]^.Params;
      SaveOrBackupRG.ItemIndex := Progs[CurRec]^.SaveOrBackupIndex;
      HotKey1.HotKey := RunMnu.Items[CurRec - 1].ShortCut;
      BatFileMemo.Lines.Assign (Progs[CurRec]^.BatFile);

      Saved := true;
      NewBtn.Enabled := CurRec < MAX_REC_NUM;
    end;
  end;

  NextBtn.Enabled := CurRec < MainForm.NumOfRecs;
  PreBtn.Enabled := CurRec > 1;

  ChangeModify (false);
end;

function TRunForm.PromptToSave : boolean;
begin
  result := true;

  if Modified then
    if ProgNamed then begin
      if MessageDlg ('The current record has been modifed.  Do you wish to SAVE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        SaveBtn.Click else result := false;
    end else if Saved then
      if MessageDlg ('The current record has been modifed and now conatins insufficient data.  Do you wish to DELETE it?',
                     mtConfirmation, [mbYes, mbNo], 0) = mrYes then
        DeleteBtn.Click
      else result := false;
end;

procedure TRunForm.SwapItems (i1, i2 : integer);

  function InValid (i : integer) : boolean;
  begin
    result := (i < 0) or (i >= MainForm.NumOfRecs);
  end;

var
  s : string;
  i : integer;
  Prog : TProg;
begin
  if (i2 = i1) or InValid (i1) or InValid (i2) then exit;

  with MainForm.RunMnu do begin
    s := CapToStr (Items[i1].Caption);
    Items[i1].Caption := StrToCap (CapToStr (Items[i2].Caption), i1 + 1);
    Items[i2].Caption := StrToCap (s, i2 + 1);

    i := Items[i1].ShortCut;
    Items[i1].ShortCut := Items[i2].ShortCut;
    Items[i2].ShortCut := i;
  end;

  inc (i1); inc (i2);
  with MainForm do begin
    Prog := Progs[i1]^;
    Progs[i1]^ := Progs[i2]^;
    Progs[i2]^ := Prog;
  end;

  if CurRec = i1 then begin
    CurRec := i2;
    ScrollBar1.Position := i2;
    RecNumEdit.Text := IntToStr (i2);
  end else if CurRec = i2 then begin
    CurRec := i1;
    ScrollBar1.Position := i1;
    RecNumEdit.Text := IntToStr (i1);
  end;
end;

procedure TRunForm.GroupBox1Click(Sender: TObject);
begin
  ProgEdit.SetFocus;
end;

procedure TRunForm.GroupBox2Click(Sender: TObject);
begin
  ParamsEdit.SetFocus;
end;

procedure TRunForm.Label1Click(Sender: TObject);
begin
  ProgNameEdit.SetFocus;
end;

procedure TRunForm.Label2Click(Sender: TObject);
begin
  HotKey1.SetFocus;
end;

procedure TRunForm.ParamsPromptCBClick(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.GroupBox5Click(Sender: TObject);
begin
  HotKey1.SetFocus;
end;

procedure TRunForm.FormKeyUp(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  {Changes modify to true if the ShortCut is
   changed}
  if ProgNamed and (CurRec <= MainForm.RunMnu.Count) then
    if MainForm.RunMnu.Items[CurRec - 1].ShortCut <> HotKey1.HotKey then
      ChangeModify (true);
end;

procedure TRunForm.cdCBClick(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.ReloadBtnClick(Sender: TObject);
begin
  ShowCurRec;
end;

procedure TRunForm.RecNumEditExit(Sender: TObject);
begin
  if RecNumEdit.Text = '' then
    RecNumEdit.Text := IntToStr (CurRec)
  else begin
    PromptToSave;
    if not ChangeRecNum (StrToInt (RecNumEdit.Text)) then
      RecNumEdit.Text := IntToStr (CurRec);
  end;
end;

procedure TRunForm.RecNumEditKeyPress(Sender: TObject; var Key: Char);
begin
  ValidateKey (Key);

  if Key = #13 then begin
    RecNumEditExit (RecNumEdit);
    Key := #0; // prevents beep
  end;
end;

procedure TRunForm.SaveOrBackupRGClick(Sender: TObject);
begin
  ChangeModify (true)
end;

procedure TRunForm.FileNameParamRGClick(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.CloseBtnClick(Sender: TObject);
begin
  Close;
end;


procedure TRunForm.FormKeyDown(Sender: TObject; var Key: Word;
  Shift: TShiftState);
begin
  if Key = VK_F1 then HelpBtn.Click;

  //KeyPrevious must be true for this to work
  if CurRec <= MainForm.RunMnu.Count then
    if MainForm.RunMnu[CurRec - 1].ShortCut <> HotKey1.Hotkey then
      ChangeModify (true);
end;

procedure TRunForm.BatEditChange(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.BatFileMemoChange(Sender: TObject);
begin
  ChangeModify (true);
end;

procedure TRunForm.BatFileMemoKeyPress(Sender: TObject; var Key: Char);
begin
  if Key = #27 then Close;
end;

procedure TRunForm.MenuItemOrderBtnClick(Sender: TObject);
var i : cardinal;
begin
  OrderForm.ListBox1.Items.Assign (MenuItemToStringList (MainForm.RunMnu));
  OrderForm.MainItem := -1;
  if OrderForm.ShowModal = mrCancel then exit;

  for i := 0 to MainForm.RunMnu.Count - 2 do
    SwapItems (i, FindCapMenuIndex (MainForm.RunMnu, OrderForm.ListBox1.Items[i]));
end;

procedure TRunForm.ScrollBar1Scroll(Sender: TObject;
  ScrollCode: TScrollCode; var ScrollPos: Integer);
begin
  {don't do anything if the user hasn't scrolled
   enough to change position}
  if ScrollBar1.Position = CurRec then exit;
  
  if not PromptToSave then
    if not PromptToChangeRec then begin
      {scroll cancled so change position to what
       it was}
      ScrollPos := CurRec;
      exit;
    end;

  ChangeRecNum (ScrollBar1.Position);
end;

end.

